/*
 *  sd.h
 *  
 *
 *  Created by Joakim Flathagen on 08.04.08.
 *  Copyright 2008 Joakim. All rights reserved.
 *
 */
#include <stdio.h>
#include <iostream>
#include <string>
#include <fstream>
#include <sstream>
#include <cstring>
#include <bitset>
#include <vector>
#include <address.h>
#include "olsr.h"

#define BLOOMFILTER_SIZE 128
#define HASH_FUNCTIONS 4

#define SD_REQUEST 1
#define SD_ADVERTISEMENT 2

using namespace std;

// This class holds the service information for one particular node
class ServiceFilter{
private:
bitset<BLOOMFILTER_SIZE> _BloomFilter;
nsaddr_t _address; 
int _numhops; // Distance to node
double _valid_time; // TBD

public:

ServiceFilter(){};
bitset<BLOOMFILTER_SIZE>* getBloomFilterPtr() {return &_BloomFilter;};
double getValidTime() {return _valid_time;};
nsaddr_t getAddress(){return _address;};
void setAddress(nsaddr_t address){_address = address;};
void setValidTime(double valid_time){_valid_time = valid_time;};

};


// SD is the main class handling all OLSR service discovery

struct SearchEntry {
string Name;
int Number;

};

class SD {
private:
OLSR *_parent;
bitset<BLOOMFILTER_SIZE> _localBloomFilter;
void printBloomFilter(bitset<BLOOMFILTER_SIZE>*);
void insertLocalServiceName(string* service);
void insertLocalServiceToBloom(string service);
bool removeLocalServiceName(string* service);

std::vector<ServiceFilter*> _foreignServices;
int _node_id;
int _num_hash_functions;
std::vector<string> _localServices; // Services provided by this host
std::vector<SearchEntry> _searchServices; // Services to search for
public:
SD(int,OLSR*);
~SD();
void setNumHashFunctions(int num){_num_hash_functions = num;};
void insertLocalService(string);
void removeLocalService(string);
void insertForeignFilter(int nsaddr_t, double& valid_time, bitset<BLOOMFILTER_SIZE>*);
void cleanupForeignFilter(double);
void resetLocalBloomFilter();
void printLocalBloomFilter();
bool searchLocalService(string);
bool searchLocalService(bitset<BLOOMFILTER_SIZE>* bloomFilter);
void createServiceRequest(string serviceName,bitset<BLOOMFILTER_SIZE>* bloomFilter, int requestNumber);
nsaddr_t searchForeignService(string);
int getCachesize();
bitset<BLOOMFILTER_SIZE>* getLocalBloomFilter();
bool uponReceivingAdvertisement(bitset<BLOOMFILTER_SIZE>* bloomFilter,int requestNumber);
protected:
void createBloomFiltervalue(string*,bitset<BLOOMFILTER_SIZE>*);
bool searchBloomFilter(string*, bitset<BLOOMFILTER_SIZE>*);
string getmd5Hash(string*);
unsigned long int ConvertHash2Int(string);

};



